// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/HPDI32/HPDI32_Linux_sdk_6.x.x_GSC_DN/api/gsc_os_device.h $
// $Rev: 30377 $
// $Date: 2014-09-18 12:09:04 -0500 (Thu, 18 Sep 2014) $

#ifndef __GSC_OS_DEVICE_H__
#define __GSC_OS_DEVICE_H__

typedef struct _os_device_t	os_device_t;

#include "dev_drv.h"
#include "gsc_common.h"
#include "gsc_os_event.h"
#include "gsc_os_key.h"
#include "gsc_os_sem.h"
#include "gsc_os_thread.h"



// #defines *******************************************************************

#define	OS_DEVICE_HANDLE(dp)			((dp)->fd)
#define	OS_DEVICE_HANDLE_IS_VALID(dp)	(OS_DEVICE_HANDLE(dp))



// data types *****************************************************************

struct _os_device_t
{
	os_key_t	key;	// First field!
	int			fd;
	char		name[64];

#if DEV_USES_IRQ

	// Event waiting is handled by putting the thread to sleep until it is
	// triggered. The thread is put in an event list before going to sleep
	// so that the triggerring mechanism knows who to resume once an event
	// is triggerred.

	struct
	{
		os_sem_t			sem;	// Must lock to access list.
		int					quit;	// Set this to abort all processing.
		os_event_node_t*	head;
		os_event_node_t*	tail;
	} events;

	// A master thread is created to handle interrupt notification in the API.
	// The thread sits in a loop where it waits for an interrupt then triggers
	// the PCI event so that the common code can check to see which interrupt
	// occurred and then respond accordingly.

	struct
	{
		int			error;	// The thread sets this if there is an error.
		int			quit;	// Set before requesting that the thread abort.
		os_thread_t	thread;	// This is the master thread.
		os_event_t*	event;	// Trigger this when we get an interrupt.
	} master;

#elif ! defined(DEV_USES_IRQ)
	#error DEV_USES_IRQ undefined. Must #include dev_drv.h.
#endif

};



// prototypes *****************************************************************

U32	os_device_close(os_device_t* device);
U32	os_device_hc_base_index(int* hc, U8* base_index);
U32	os_device_open(int index, const char* name, os_device_t* device);



#endif
