// This software is covered by the GNU GENERAL PUBLIC LICENSE (GPL).
// Any Device - Driver - Linux
// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/HPDI32/HPDI32_Linux_sdk_6.x.x_GSC_DN/driver/gsc_driver.h $
// $Rev: 23671 $
// $Date: 2013-10-31 16:23:27 -0500 (Thu, 31 Oct 2013) $

#ifndef __GSC_DRIVER_H__
#define __GSC_DRIVER_H__

#include <asm/types.h>
#include <linux/ioctl.h>

#define	GSC_DATA_TYPES_NOT_NEEDED
#include "gsc_common.h"



/* #defines	**************************************************************/

#define	GSC_DRV_INTERFACE_VERSION		2
// 2	Updated for the 2.6.26 kernel.
// 1	Initial release.

// IOCTL codes.
#define	GSC_IOCTL_VERSION_GET			_IOR( 'g', 0, gsc_drv_version_t)

#define	GSC_IOCTL_BAR_INFO_GET			_IOWR('g', 1, gsc_drv_bar_t)

#define	GSC_IOCTL_REG_MOD				_IOWR('g', 2, gsc_drv_reg_t)
#define	GSC_IOCTL_REG_READ				_IOWR('g', 3, gsc_drv_reg_t)
#define	GSC_IOCTL_REG_WRITE				_IOWR('g', 4, gsc_drv_reg_t)

#define	GSC_IOCTL_MEM_ALLOC				_IOWR('g', 5, gsc_drv_mem_t)
#define	GSC_IOCTL_MEM_FREE				_IOWR('g', 6, gsc_drv_mem_t)
#define	GSC_IOCTL_MEM_INFO				_IOWR('g', 7, gsc_drv_mem_t)

#define	GSC_IOCTL_IRQ_DISABLE			_IOWR('g', 8, gsc_drv_irq_t)
#define	GSC_IOCTL_IRQ_ENABLE			_IOWR('g', 9, gsc_drv_irq_t)
#define	GSC_IOCTL_IRQ_NOTIFY			_IOWR('g',10, gsc_drv_irq_t)
#define	GSC_IOCTL_IRQ_STATUS			_IOR( 'g',11, gsc_drv_irq_t)
#define	GSC_IOCTL_IRQ_WAIT				_IOWR('g',12, gsc_drv_irq_t)
#define	GSC_IOCTL_IRQ_WAIT_CANCEL		_IOWR('g',13, gsc_drv_irq_t)

#define	GSC_IOCTL_DMA_ABORT				_IOWR('g',14, gsc_drv_dma_t)
#define	GSC_IOCTL_DMA_CLOSE				_IOWR('g',15, gsc_drv_dma_t)
#define	GSC_IOCTL_DMA_OPEN				_IOWR('g',16, gsc_drv_dma_t)
#define	GSC_IOCTL_DMA_STATUS			_IOWR('g',17, gsc_drv_dma_t)
#define	GSC_IOCTL_DMA_XFER				_IOWR('g',18, gsc_drv_dma_t)

// This is the maximum number of memory allocations per process per device.
#define	GSC_MEM_ALLOC_MAX				32

// These are used with the IRQ IOCTL services. All are independent.
#define	GSC_IRQ_PCI						0x00000001
#define	GSC_IRQ_DMA0					0x00000002
#define	GSC_IRQ_DMA1					0x00000004
#define	GSC_IRQ_LOCAL					0x00000008
#define	GSC_IRQ_MASK					0x0000000F

// These are used with the DMA OPEN IOCTL service.
#define	GSC_DMA_OPEN_PRI_MASK			0x00000003
#define	GSC_DMA_OPEN_PRI_CH_0			0x00000001	/* Channel 0 */
#define	GSC_DMA_OPEN_PRI_CH_1			0x00000002	/* Channel 1 */
#define	GSC_DMA_OPEN_PRI_SHARE			0x00000003	/* Shared priority */

#define	GSC_DMA_OPEN_SIZE_MASK			0x0000000C
#define	GSC_DMA_OPEN_SIZE_8_BITS		0x00000004	/* 8-bits */
#define	GSC_DMA_OPEN_SIZE_16_BITS		0x00000008	/* 16-bits */
#define	GSC_DMA_OPEN_SIZE_32_BITS		0x0000000C	/* 32-bits */

#define	GSC_DMA_OPEN_MEM_MASK			0x00000030
#define	GSC_DMA_OPEN_MEM_BLOCK			0x00000010	/* single fixed block */
#define	GSC_DMA_OPEN_MEM_SGL			0x00000020	/* scatter gather list */

#define	GSC_DMA_OPEN_XFER_MASK			0x000000C0
#define	GSC_DMA_OPEN_XFER_DEMAND		0x00000040	/* demand mode */
#define	GSC_DMA_OPEN_XFER_FORCE			0x00000080	/* forced transfer */

#define	GSC_DMA_OPEN_LADRS_MASK			0x00000300
#define	GSC_DMA_OPEN_LADRS_FIXED		0x00000100	/* FIFO like */
#define	GSC_DMA_OPEN_LADRS_INC			0x00000200	/* memory like */

#define	GSC_DMA_OPEN_BTERM_INPUT		0x00010000	/* yes or no */
#define	GSC_DMA_OPEN_EOT_PIN			0x00020000	/* yes or no */
#define	GSC_DMA_OPEN_LOCAL_BURST		0x00040000	/* yes or no */
#define	GSC_DMA_OPEN_READY_INPUT		0x00080000	/* yes or no */
#define	GSC_DMA_OPEN_STOP_XFER_MODE		0x00100000	/* yes or no */
#define	GSC_DMA_OPEN_WR_INVAL_MODE		0x00200000	/* yes or no */

// A DIRECTION must be specified with each transfer.
// The DIRECTION is reported as status while the transfer is active.
#define	GSC_DMA_XFER_DIR_TO_DEVICE		0x04000000	/* A DIR is required. */
#define	GSC_DMA_XFER_DIR_TO_HOST		0x08000000	/* A DIR is required. */

#define	GSC_DMA_STATUS_BUSY				0x40000000	/* STATUS */
#define	GSC_DMA_STATUS_IDLE				0x80000000	/* STATUS */



/* typedefs	**************************************************************/

typedef struct
{
	__u32	status;			// Results of the  IOCTL request.
	__u8	index;			// BARX: 0 ... 5
	__u8	mmap;			// Is the region mmap()'d?
	__u32	reg;			// Value read from the PCI register.
	__u32	size;			// Size in bytes.
	__u32	mmap_offset;	// Given to mmap().
	__u32	mmap_size;		// Given to mmap() and munmap().
	__u32	mmap_start;		// Given to munmap().
	__u32	mmap_inset;		// Regs are at mmap() + mmap_inset.
} gsc_drv_bar_t;

typedef struct
{
	__u32	status;		// Results of the IOCTL request.
	__u32	index;		// Always needed.

	union
	{
		struct
		{
			__u32	flags;			// GSC_DMA_OPEN_XXX
			__u32	wait_states;	// 0-15
		} open;						// For open requests only.

		struct
		{
			__u32	flags;
		} status;					// For status requests only.

		struct
		{
			__u32			flags;
			__u32			bytes;
			__u32			local_adrs;
			__u32			phys_adrs;
			unsigned long	user_adrs;
		} xfer;						// For DMA transfers.
	} u;
} gsc_drv_dma_t;

typedef struct
{
	__u32	status;		// Results of the  IOCTL request.
	__s8	bar;		// The Base Address Register (-1 for PCI registers).
	__u8	size;		// In bytes.
	__u16	offset;		// In bytes from the base of the region.
	__u32	value;		// The value read, to write or to apply.
	__u32	mask;		// For read-modify-write operations only.
} gsc_drv_reg_t;

typedef struct
{
	__u32	status;			// Results of the IOCTL request.
	__u32	enable;			// Enable/disable these.
	__u32	notify;			// Notify application of these interrupts.
	__u32	triggered;		// Of those requested, these have been triggered.
	__u32	wait_seconds;	// 60 seconds max, 1 second min - SILENT LIMITS!
	__u32	wait_cancel;	// Cancel wait? If not, then cancel is cancelled.
} gsc_drv_irq_t;

typedef struct
{
	__u32	status;			// Results of the IOCTL request.
	__u8	index;			// Set by driver.
	__u8	mmapped;		// Is mapped to user space?
	__u32	size;			// The size desired/granted in bytes.
	__u32	mmap_offset;	// The offset given to mmap().
	__u32	dma_address;	// The address used when doing DMA.
} gsc_drv_mem_t;

typedef struct
{
	__u32	status;			// Results of the  IOCTL request.
	__u8	api;			// Interface version.
	__u8	version[23];	// Executable image.
	__u8	name[16];		// Product name.
} gsc_drv_version_t;



#endif
