// This software is covered by the GNU GENERAL PUBLIC LICENSE (GPL).
// Any Device - Driver - Linux
// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/HPDI32/HPDI32_Linux_sdk_6.x.x_GSC_DN/driver/pid.c $
// $Rev: 23671 $
// $Date: 2013-10-31 16:23:27 -0500 (Thu, 31 Oct 2013) $

#include "main.h"



/******************************************************************************
*
*	Function:	pid_add
*
*	Purpose:
*
*		Add the current pid to our list.
*
*	Arguments:
*
*		dev	The data for the device of interest.
*
*	Returned:
*
*		0	All went well.
*		<0	An error code for the problem seen.
*
******************************************************************************/

int pid_add(dev_data_t* dev)
{
	int	i;
	int	status;

	for (;;)	// A convenience loop.
	{
		/* Look for an existing entry for the current id. */

		for (i = ARRAY_ELEMENTS(dev->pid); i--;)
		{
			if ((dev->pid[i]) && (dev->pid[i]->pid == CURRENT_PROCESS_ID))
				break;
		}

		if (i >= 0)
		{
			status	= 0;
			break;
		}

		/* Look for an available entry if the pid is a new one. */

		for (i = ARRAY_ELEMENTS(dev->pid); i--;)
		{
			if (dev->pid[i] == NULL)
				break;
		}

		if (i < 0)
		{
			status	= -EBUSY;
			break;
		}

		dev->pid[i]	= kmalloc(sizeof(dev_pid_t), GFP_KERNEL);

		if (dev->pid[i] == NULL)
		{
			/* We didn't get the needed memory. */
			status	= -ENOMEM;
			printk("<1>%s: kmalloc() failure.\n", DRV_DEV_DEVICE_NAME_LOWER);
			break;
		}

		/* Account for the entry. */
		memset(dev->pid[i], 0, sizeof(dev_pid_t));
		dev->pid[i]->pid	= CURRENT_PROCESS_ID;
		WAIT_QUEUE_HEAD_INIT(&dev->pid[i]->queue);
		status	= 0;
		break;
	}

	return(status);
}



/******************************************************************************
*
*	Function:	pid_locate
*
*	Purpose:
*
*		Locate the device data for the current process.
*
*	Arguments:
*
*		dev		The data for the device being accessed.
*
*	Returned:
*
*		NULL	It wasn't found.
*		else	The structure requested.
*
******************************************************************************/

dev_pid_t* pid_locate(dev_data_t* dev)
{
	int			i;
	dev_pid_t*	pid	= NULL;

	/* Look for an existing entry for the current process. */

	for (i = ARRAY_ELEMENTS(dev->pid); i--;)
	{
		if ((dev->pid[i]) && (dev->pid[i]->pid == CURRENT_PROCESS_ID))
		{
			pid	= dev->pid[i];
			break;
		}
	}

	return(pid);
}



/******************************************************************************
*
*	Function:	pid_remove
*
*	Purpose:
*
*		Remove the given pid from our list.
*
*	Arguments:
*
*		pid		The Procedd IDentifier to remove.
*
*	Returned:
*
*		0		All went well.
*		<0		An error code for the problem seen.
*
******************************************************************************/

int pid_remove(dev_data_t* dev)
{
	int			i;
	dev_pid_t*	pid;
	int			status	= -EINVAL;

	for (i = ARRAY_ELEMENTS(dev->pid); i--;)
	{
		if ((dev->pid[i]) && (dev->pid[i]->pid == CURRENT_PROCESS_ID))
		{
			pid			= dev->pid[i];
			dev->pid[i]	= NULL;

			if (pid->queued)
				EVENT_RESUME_IRQ(&pid->queue, pid->condition);

			kfree(pid);
			status	= 0;
			break;
		}
	}

	return(status);
}



