// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/HPDI32/HPDI32_Linux_sdk_6.x.x_GSC_DN/gpio/dout/dio.c $
// $Rev: 30432 $
// $Date: 2014-10-07 14:50:49 -0500 (Tue, 07 Oct 2014) $

#include <stdio.h>
#include <unistd.h>

#include "main.h"



//*****************************************************************************
static void _show_u8(U8 mask, U8 value)
{
	int	bit		= 0x80;
	int	i;

	for (i = 0; i < 8; i++, bit >>= 1)
	{
		if ((bit & mask) == 0)
			printf("-");
		else if (bit & value)
			printf("1");
		else
			printf("0");
	}
}



//*****************************************************************************
static int _output(void* dev, U8 ctrl_mask, U8 ctrl, U32 data_mask, U32 data)
{
	int	errs	= 0;
	U32	status	= GSC_SUCCESS;

	printf("    ");
	_show_u8(ctrl_mask, ctrl);
	printf("  ");
	_show_u8((U8) ((data_mask >> 24) & 0xFF), (U8) ((data  >> 24) & 0xFF));
	printf(" ");
	_show_u8((U8) ((data_mask >> 16) & 0xFF), (U8) ((data  >> 16) & 0xFF));
	printf(" ");
	_show_u8((U8) ((data_mask >> 8) & 0xFF), (U8) ((data  >> 8) & 0xFF));
	printf(" ");
	_show_u8((U8) (data_mask & 0xFF), (U8) (data  & 0xFF));

	if (ctrl_mask)
	{
		status	= hpdi32_gpio_ctrl_tx(dev, ctrl);
		errs	+= (status == GSC_SUCCESS) ? 0 : 1;
	}

	if (data_mask)
	{
		status	= hpdi32_gpio_data_tx(dev, data);
		errs	+= (status == GSC_SUCCESS) ? 0 : 1;
	}

	printf("%s\n", errs ? "  FAIL <---" : "");
	usleep(250000);
	return(errs);
}



//*****************************************************************************
int dio_out(void* dev)
{
	U8	cap			= 0;
	U8	ctrl;
	U8	ctrl_mask;
	U32	data;
	U32	data_mask;
	int	errs		= 0;
	int	i;

	for (;;)	// A convenience loop
	{
		errs	+= dev_gpio_dev_cap_get(dev, -1, 1, &cap);
		errs	+= dev_init(dev, -1, 1);

		errs	+= dev_gpio_config_set(dev, -1, 1, 0xFF, 0xFF, 0, 0);

		label_text("GPIO Output");
		printf("\n");

		if (cap & HPDI32_GPIO_CAP_CTRL_0_6)
			ctrl_mask	= 0x7F;
		else if (cap & HPDI32_GPIO_CAP_CTRL_0_5)
			ctrl_mask	= 0x3F;
		else
			ctrl_mask	= 0;

		if (cap & HPDI32_GPIO_CAP_DATA_EXP)
			data_mask	= 0xFFFFFFFF;
		else if (cap & HPDI32_GPIO_CAP_DATA_SIM)
			data_mask	= 0xFFFFFFFF;
		else
			data_mask	= 0;

		if ((ctrl_mask == 0x00) && (data_mask == 0x00000000))
		{
			printf(	"SKIPPED  (No GPIO capabilities.)\n");
			break;
		}

		printf("\n");
		printf("    Control   Data (Hi to Low)\n");
		printf("    76543210  76543210 76543210 76543210 76543210\n");
		printf("    ========  ======== ======== ======== ========\n");

		errs	+= _output(dev, ctrl_mask, 0x00, data_mask, 0x00000000);
		errs	+= _output(dev, ctrl_mask, 0xFF, data_mask, 0xFFFFFFFF);
		errs	+= _output(dev, ctrl_mask, 0x00, data_mask, 0x00000000);
		errs	+= _output(dev, ctrl_mask, 0xFF, data_mask, 0xFFFFFFFF);
		errs	+= _output(dev, ctrl_mask, 0x00, data_mask, 0x00000000);

		ctrl	= 0;
		data	= 0;

		for (i = 0; i < 32; i++)
		{
			if (ctrl_mask)
			{
				ctrl	= (ctrl << 1) & ctrl_mask;
				ctrl	= ctrl ? ctrl : 1;
			}

			if (data_mask)
				data	= 0x01010101 << (i % 8);

			errs	+= _output(dev, ctrl_mask, ctrl, data_mask, data);
		}

		errs	+= _output(dev, ctrl_mask, 0x00, data_mask, 0x00000000);
		errs	+= _output(dev, ctrl_mask, 0xFF, data_mask, 0xFFFFFFFF);
		errs	+= _output(dev, ctrl_mask, 0x00, data_mask, 0x00000000);
		errs	+= _output(dev, ctrl_mask, 0xFF, data_mask, 0xFFFFFFFF);
		errs	+= _output(dev, ctrl_mask, 0x00, data_mask, 0x00000000);

		printf("\n");
		dev_gpio_dev_release(dev, -1, 1);
		errs	+= dev_init(dev, -1, 1);
		break;
	}

	return(errs);
}


