// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/HPDI32/HPDI32_Linux_sdk_6.x.x_GSC_DN/gpio/lib/ctrl_def.c $
// $Rev: 30430 $
// $Date: 2014-10-07 14:49:48 -0500 (Tue, 07 Oct 2014) $

#include <string.h>

#include "main.h"




// variables ******************************************************************

static const struct
{
	int	which;
	int	bsr_mask;
} _cfg_src[7]	=
{
	// which				bsr_mask
	{ HPDI32_WHICH_GPIO_0_,	D1	},	// Command 1: GPIO 0, Line Valid
	{ HPDI32_WHICH_GPIO_1_,	D2	},	// Command 2: GPIO 1, Status Valid
	{ HPDI32_WHICH_GPIO_2_,	D3	},	// Command 3: GPIO 2, Rx Ready
	{ HPDI32_WHICH_GPIO_3_,	D4	},	// Command 4: GPIO 3, Tx Data Ready
	{ HPDI32_WHICH_GPIO_4_,	D5	},	// Command 5: GPIO 4, Tx Enabled
	{ HPDI32_WHICH_GPIO_5_,	D6	},	// Command 6: GPIO 5, Rx Enabled,
	{ HPDI32_WHICH_GPIO_6_, D0	}	// Command 0: GPIO 6, Frame Valid
};



//*****************************************************************************
U32 ctrl_def_cap_get(void* dev, U8* cap, U8* cfg_mask)
{
	U8				cap_tmp	= 0;
	U8				cfg_mask_tmp	= 0;
	unsigned long	gpio_0_5;
	unsigned long	gpio_6;
	U32				status;
	U32				sts;

	gpio_0_5	= HPDI32_MISC_FEATURES_ABSENT;
	status		= HPDI32_MISC_FEATURES__GPIO_0_5(dev, &gpio_0_5);

	gpio_6		= HPDI32_MISC_FEATURES_ABSENT;
	sts			= HPDI32_MISC_FEATURES__GPIO_6(dev, &gpio_6);
	status		= (status == GSC_SUCCESS) ? sts : status;

	if (gpio_6	== HPDI32_MISC_FEATURES_PRESENT)
	{
		// GPIO 0 to 6 are supported.
		// All support Tx and Rx.
		cap_tmp			= HPDI32_GPIO_CAP_CTRL_0_6
						| HPDI32_GPIO_CAP_CTRL_TX
						| HPDI32_GPIO_CAP_CTRL_RX;
		cfg_mask_tmp	= 0x7F;
	}
	else if (gpio_0_5 == HPDI32_MISC_FEATURES_PRESENT)
	{
		// GPIO 0 to 5 are supported.
		// All support Tx and Rx.
		cap_tmp			= HPDI32_GPIO_CAP_CTRL_0_5
						| HPDI32_GPIO_CAP_CTRL_TX
						| HPDI32_GPIO_CAP_CTRL_RX;
		cfg_mask_tmp	= 0x3F;
	}
	else
	{
		cap_tmp			= 0;
		cfg_mask_tmp	= 0;
	}

	if (cap)
		cap[0]		= cap_tmp;

	if (cfg_mask)
		cfg_mask[0]	= cfg_mask_tmp;

	return(status);
}



//*****************************************************************************
U32 ctrl_def_enable_get(void* dev, U8* cfg_en)
{
	U8				bit;
	U8				cfg_en_tmp	= 0;
	U8				cfg_mask	= 0;
	int				i;
	unsigned long	state;
	U32				status;
	U32				sts;

	for (;;)	// A convenience loop.
	{
		status	= gpio_dev_cap_get(dev, NULL, &cfg_mask, NULL);

		if (status != GSC_SUCCESS)
			break;

		for (i = 0; i < 7; i++)
		{
			bit	= 0x1 << i;

			if ((bit & cfg_mask) == 0)
				continue;

			state	= HPDI32_CABLE_COMMAND_MODE_FLOW_CONTROL;
			sts		= HPDI32_CABLE_COMMAND_MODE__GET(dev, _cfg_src[i].which, &state);
			status	= (status == GSC_SUCCESS) ? sts : status;

			if ((state == HPDI32_CABLE_COMMAND_MODE_GPIO_OUT_LOW) ||
				(state == HPDI32_CABLE_COMMAND_MODE_GPIO_OUT_HI) ||
				(state == HPDI32_CABLE_COMMAND_MODE_GPIO_IN))
			{
				cfg_en_tmp	|= bit;
			}
		}

		if (cfg_en)
			cfg_en[0]	= cfg_en_tmp;

		break;
	}

	return(status);
}



//*****************************************************************************
U32 ctrl_def_enable_set(void* dev, U8 cfg_en, U8 cfg_dir, U8 io_bin)
{
	U8	bit;
	U8	cfg_mask	= 0;
	int	i;
	U32	status;
	U32	sts;

	for (;;)	// A convenience loop.
	{
		status	= gpio_dev_cap_get(dev, NULL, &cfg_mask, NULL);

		if (status != GSC_SUCCESS)
			break;

		for (i = 0; i < 7; i++)
		{
			bit	= 0x1 << i;

			if ((bit & cfg_mask) == 0)
				continue;

			if ((bit & cfg_en) == 0)
			{
				// Disable this GPIO line: restore its default functionality
				sts		= HPDI32_CABLE_COMMAND_MODE__FC(dev, _cfg_src[i].which);
				status	= (status == GSC_SUCCESS) ? sts : status;
			}
			else if ((bit & cfg_dir) == 0)
			{
				// Enable the GPIO line: input
				sts		= HPDI32_CABLE_COMMAND_MODE__GPIO_IN(dev, _cfg_src[i].which);
				status	= (status == GSC_SUCCESS) ? sts : status;
			}
			else if (bit & io_bin)
			{
				// Enable the GPIO line: output high
				sts		= HPDI32_CABLE_COMMAND_MODE__GPIO_HI(dev, _cfg_src[i].which);
				status	= (status == GSC_SUCCESS) ? sts : status;
			}
			else
			{
				// Enable the GPIO line: output low
				sts		= HPDI32_CABLE_COMMAND_MODE__GPIO_LOW(dev, _cfg_src[i].which);
				status	= (status == GSC_SUCCESS) ? sts : status;
			}
		}

		break;
	}

	return(status);
}



//*****************************************************************************
U32 ctrl_def_dir_out_get(void* dev, U8* cfg_dir)
{
	U8				bit;
	U8				cfg_dir_tmp	= 0;
	U8				cfg_en		= 0;
	U8				cfg_mask	= 0;
	int				i;
	unsigned long	state;
	U32				status;
	U32				sts;

	for (;;)	// A convenience loop.
	{
		status	= gpio_dev_cap_get(dev, NULL, &cfg_mask, NULL);

		if (status != GSC_SUCCESS)
			break;

		status	= ctrl_def_enable_get(dev, &cfg_en);

		if (status != GSC_SUCCESS)
			break;

		for (i = 0; i < 7; i++)
		{
			bit	= 0x1 << i;

			if ((bit & cfg_mask & cfg_en) == 0)
				continue;

			state	= HPDI32_CABLE_COMMAND_MODE_FLOW_CONTROL;
			sts		= HPDI32_CABLE_COMMAND_MODE__GET(dev, _cfg_src[i].which, &state);
			status	= (status == GSC_SUCCESS) ? sts : status;

			if ((state == HPDI32_CABLE_COMMAND_MODE_GPIO_OUT_LOW) ||
				(state == HPDI32_CABLE_COMMAND_MODE_GPIO_OUT_HI))
			{
				cfg_dir_tmp	|= bit;
			}
		}

		if (cfg_dir)
			cfg_dir[0]	= cfg_dir_tmp;

		break;
	}

	return(status);
}



//*****************************************************************************
U32 ctrl_def_dir_out_set(void* dev, U8 cfg_dir, U8 io_bin)
{
	U8	bit;
	U8	cfg_en		= 0;
	U8	cfg_mask	= 0;
	int	i;
	U32	status;
	U32	sts;

	for (;;)	// A convenience loop.
	{
		status	= gpio_dev_cap_get(dev, NULL, &cfg_mask, NULL);

		if (status != GSC_SUCCESS)
			break;

		status	= ctrl_def_enable_get(dev, &cfg_en);

		if (status != GSC_SUCCESS)
			break;

		for (i = 0; i < 7; i++)
		{
			bit	= 0x1 << i;

			if ((bit & cfg_mask & cfg_en) == 0)
				continue;

			if ((bit & cfg_dir) == 0)
			{
				// Enable the GPIO line: input
				sts		= HPDI32_CABLE_COMMAND_MODE__GPIO_IN(dev, _cfg_src[i].which);
				status	= (status == GSC_SUCCESS) ? sts : status;
			}
			else if (bit & io_bin)
			{
				// Enable the GPIO line: output high
				sts		= HPDI32_CABLE_COMMAND_MODE__GPIO_HI(dev, _cfg_src[i].which);
				status	= (status == GSC_SUCCESS) ? sts : status;
			}
			else
			{
				// Enable the GPIO line: output low
				sts		= HPDI32_CABLE_COMMAND_MODE__GPIO_LOW(dev, _cfg_src[i].which);
				status	= (status == GSC_SUCCESS) ? sts : status;
			}
		}

		break;
	}

	return(status);
}



//*****************************************************************************
U32 ctrl_def_rx(void* dev, U8* io_bin)
{
	U8	bit;
	U32	bsr			= 0;
	U8	cfg_en		= 0;
	U8	cfg_mask	= 0;
	U8	io_bin_tmp	= 0;
	int	i;
	U32	status;

	for (;;)	// A convcenience loop.
	{
		status	= gpio_dev_cap_get(dev, NULL, &cfg_mask, NULL);

		if (status != GSC_SUCCESS)
			break;

		status	= ctrl_def_enable_get(dev, &cfg_en);

		if (status != GSC_SUCCESS)
			break;

		status	= hpdi32_reg_read(dev, HPDI32_BSR, &bsr);

		if (status != GSC_SUCCESS)
			break;

		for (i = 0; i < 7; i++)
		{
			bit	= 0x1 << i;

			if ((bit & cfg_mask & cfg_en) == 0)
				continue;

			if (bsr & _cfg_src[i].bsr_mask)
				io_bin_tmp	|= bit;
		}

		if (io_bin)
			io_bin[0]	= io_bin_tmp;

		break;
	}

//printf("CTRL: cfg_mask 0x%02lX, cfg_en 0x%02lX, io_bin_tmp 0x%02lX\n",
//	   (long) cfg_mask, (long) cfg_en, (long) io_bin_tmp);

	return(status);
}



//*****************************************************************************
U32 ctrl_def_tx(void* dev, U8 io_bin)
{
	U8				bit;
	U8				cfg_en		= 0;
	U8				cfg_mask	= 0;
	int				i;
	unsigned long	state		= 0;
	U32				status		= GSC_SUCCESS;
	U32				sts;

	for (;;)	// A convcenience loop.
	{
		status	= gpio_dev_cap_get(dev, NULL, &cfg_mask, NULL);

		if (status != GSC_SUCCESS)
			break;

		status	= ctrl_def_enable_get(dev, &cfg_en);

		if (status != GSC_SUCCESS)
			break;

		for (i = 0; i < 7; i++)
		{
			bit	= 0x1 << i;

			if ((bit & cfg_mask & cfg_en) == 0)
				continue;

			state	= HPDI32_CABLE_COMMAND_MODE_FLOW_CONTROL;
			sts		= HPDI32_CABLE_COMMAND_MODE__GET(dev, _cfg_src[i].which, &state);
			status	= (status == GSC_SUCCESS) ? sts : status;

			switch (state)
			{
				default:

					continue;

				case HPDI32_CABLE_COMMAND_MODE_GPIO_OUT_LOW:
				case HPDI32_CABLE_COMMAND_MODE_GPIO_OUT_HI:

					break;
			}

			if (io_bin & bit)
				sts	= HPDI32_CABLE_COMMAND_MODE__GPIO_HI(dev, _cfg_src[i].which);
			else
				sts	= HPDI32_CABLE_COMMAND_MODE__GPIO_LOW(dev, _cfg_src[i].which);

			status	= (status == GSC_SUCCESS) ? sts : status;
		}

		break;
	}

	return(status);
}


