// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/HPDI32/HPDI32_Linux_sdk_6.x.x_GSC_DN/gpio/lib/data_sim.c $
// $Rev: 30430 $
// $Date: 2014-10-07 14:49:48 -0500 (Tue, 07 Oct 2014) $

#include <string.h>
#include <unistd.h>

#include "main.h"



// #defines *******************************************************************

#define	TX_AE_DEF			2



//*****************************************************************************
static U32 _sim_config(void* dev)
{
	U32	status	= GSC_SUCCESS;
	U32	sts;

	// Miscelaneous

	sts		= HPDI32_MISC_TX_RX_TRI_STATE__YES(dev);
	status	= (status == GSC_SUCCESS) ? sts : status;

	// Receiver

	sts		= HPDI32_IO_DATA_SIZE__RX_32(dev);
	status	= (status == GSC_SUCCESS) ? sts : status;

	sts		= HPDI32_FIFO_RESET__RX_RESET(dev);
	status	= (status == GSC_SUCCESS) ? sts : status;

	sts		= HPDI32_IO_MODE__RX_PIO(dev);
	status	= (status == GSC_SUCCESS) ? sts : status;

	sts		= HPDI32_IO_OVERLAP_ENABLE__RX_NO(dev);
	status	= (status == GSC_SUCCESS) ? sts : status;

	// Transmitter

	sts		= HPDI32_TX_CLOCK_DIVIDER__SET(dev, 0);
	status	= (status == GSC_SUCCESS) ? sts : status;

	sts		= HPDI32_TX_LINE_VALID_OFF_COUNT__SET(dev, 0);
	status	= (status == GSC_SUCCESS) ? sts : status;

	sts		= HPDI32_TX_LINE_VALID_ON_COUNT__SET(dev, 0);
	status	= (status == GSC_SUCCESS) ? sts : status;

	sts		= HPDI32_TX_STATUS_VALID_COUNT__SET(dev, 0);
	status	= (status == GSC_SUCCESS) ? sts : status;

	sts		= HPDI32_TX_AUTO_START__NO(dev);
	status	= (status == GSC_SUCCESS) ? sts : status;

	sts		= HPDI32_TX_AUTO_STOP__NO(dev);
	status	= (status == GSC_SUCCESS) ? sts : status;

	sts		= HPDI32_TX_FLOW_CONTROL__START(dev);
	status	= (status == GSC_SUCCESS) ? sts : status;

	sts		= HPDI32_TX_REMOTE_THROTTLE__DISABLE(dev);
	status	= (status == GSC_SUCCESS) ? sts : status;

	sts		= HPDI32_FIFO_RESET__TX_RESET(dev);
	status	= (status == GSC_SUCCESS) ? sts : status;

	sts		= HPDI32_IO_DATA_SIZE__TX_32(dev);
	status	= (status == GSC_SUCCESS) ? sts : status;

	sts		= HPDI32_FIFO_ALMOST_LEVEL__TX_AE_SET(dev, TX_AE_DEF);
	status	= (status == GSC_SUCCESS) ? sts : status;

	sts		= HPDI32_IO_MODE__TX_PIO(dev);
	status	= (status == GSC_SUCCESS) ? sts : status;

	sts		= HPDI32_IO_OVERLAP_ENABLE__TX_NO(dev);
	status	= (status == GSC_SUCCESS) ? sts : status;

	return(status);
}



//*****************************************************************************
U32 data_sim_cap_get(U8* data_cap, U8* cfg_mask)
{
	U8	cfg_mask_tmp	= 0;
	U8	data_cap_tmp	= 0;
	U32	status			= GSC_SUCCESS;

	// If GPIO 0-6 are supported ...
	// An external clock is required for Rx operations.
	// Frame Valid: is configured as GPIO
	// Line Valid: is configured as GPIO
	// Status Valid: is configured as GPIO

	// If GPIO 0-5 are supported ...
	// An external clock is required for Rx operations.
	// Frame Valid: Tx: operates as normal.
	// Frame Valid: Rx: must be held high externally.
	// Line Valid: is configured as GPIO
	// Status Valid: is configured as GPIO

	// If no GPIO lines are supported.
	// An external clock is required for Rx operations.
	// Frame Valid: Tx: operates as normal.
	// Frame Valid: Rx: must be held high externally.
	// Line Valid: Tx: on/off counts set be zero.
	// Line Valid: Rx: must be held high externally.
	// Status Valid: Tx: count set be zero.
	// Status Valid: Rx: must be held high externally.

	data_cap_tmp	= HPDI32_GPIO_CAP_DATA_SIM
					| HPDI32_GPIO_CAP_DATA_RX
					| HPDI32_GPIO_CAP_DATA_TX;
	cfg_mask_tmp	= HPDI32_GPIO_CFG_D32;

	if (data_cap)
		data_cap[0]	= data_cap_tmp;

	if (cfg_mask)
		cfg_mask[0]	= cfg_mask_tmp;

	return(status);
}



//*****************************************************************************
U32 data_sim_enable_get(void* dev, U8* cfg_en)
{
	U8				cfg_en_tmp	= 0;
	unsigned long	rx_en;
	U32				status		= GSC_SUCCESS;
	U32				sts;
	unsigned long	tx_en;

	// GPIO uses the receiver and the transmitter.
	tx_en		= HPDI32_TX_ENABLE_NO;
	sts			= HPDI32_TX_ENABLE__GET(dev, &tx_en);
	status		= (status == GSC_SUCCESS) ? sts : status;

	rx_en		= HPDI32_RX_ENABLE_NO;
	sts			= HPDI32_RX_ENABLE__GET(dev, &rx_en);
	status		= (status == GSC_SUCCESS) ? sts : status;

	if ((tx_en == HPDI32_RX_ENABLE_YES) || (rx_en == HPDI32_RX_ENABLE_YES))
		cfg_en_tmp	= HPDI32_GPIO_CFG_D32;
	else
		cfg_en_tmp	= 0;

	if (cfg_en)
		cfg_en[0]	= cfg_en_tmp;

	return(status);
}



//*****************************************************************************
U32 data_sim_enable_set(void* dev, U8 cfg_en, U8 cfg_dir, U32 io_bin)
{
	U32	status	= GSC_SUCCESS;
	U32	sts;

	// GPIO uses the receiver and the transmitter.

	if ((cfg_en & HPDI32_GPIO_CFG_D32) == 0)
	{
		// GPIO: disable
		// Tx: disable
		// Rx: disable
		sts		= HPDI32_TX_ENABLE__NO(dev);
		status	= (status == GSC_SUCCESS) ? sts : status;

		sts		= HPDI32_RX_ENABLE__NO(dev);
		status	= (status == GSC_SUCCESS) ? sts : status;
	}
	else if (cfg_dir & HPDI32_GPIO_CFG_D32)
	{
		// GPIO: enable
		// Tx: enable
		// Rx: disable
		sts		= _sim_config(dev);
		status	= (status == GSC_SUCCESS) ? sts : status;

		sts		= HPDI32_RX_ENABLE__NO(dev);
		status	= (status == GSC_SUCCESS) ? sts : status;

		sts		= HPDI32_TX_ENABLE__YES(dev);
		status	= (status == GSC_SUCCESS) ? sts : status;

		sts		= data_sim_tx(dev, io_bin);
		status	= (status == GSC_SUCCESS) ? sts : status;
	}
	else
	{
		// GPIO: enable
		// Tx: disable
		// Rx: enable
		sts		= _sim_config(dev);
		status	= (status == GSC_SUCCESS) ? sts : status;

		sts		= HPDI32_TX_ENABLE__NO(dev);
		status	= (status == GSC_SUCCESS) ? sts : status;

		sts		= HPDI32_RX_ENABLE__YES(dev);
		status	= (status == GSC_SUCCESS) ? sts : status;

		sts		= HPDI32_FIFO_RESET__RX_YES(dev);
		status	= (status == GSC_SUCCESS) ? sts : status;
	}

	return(status);
}



//*****************************************************************************
U32 data_sim_dir_out_get(void* dev, U8* cfg_dir)
{
	U8				cfg_dir_tmp	= 0;
	U32				status		= GSC_SUCCESS;
	U32				sts;
	unsigned long	tx_en;

	// GPIO uses the receiver and the transmitter.
	// Output may be enabled or disable.

	sts		= HPDI32_TX_ENABLE__GET(dev, &tx_en);
	status	= (status == GSC_SUCCESS) ? sts : status;

	if (tx_en == HPDI32_TX_ENABLE_YES)
		cfg_dir_tmp	= HPDI32_GPIO_CFG_D32;
	else
		cfg_dir_tmp	= 0;

	if (cfg_dir)
		cfg_dir[0]	= cfg_dir_tmp;

	return(status);
}



//*****************************************************************************
U32 data_sim_dir_out_set(void* dev, U8 cfg_dir, U32 io_bin)
{
	U32	status	= GSC_SUCCESS;
	U32	sts;

	// GPIO uses the receiver and the transmitter.
	// Tx: may be enabled or disable.
	// Rx: may be enabled or disable.

	if (cfg_dir & HPDI32_GPIO_CFG_D32)
	{
		// GPIO: enable
		// Tx: enable
		// Rx: disabled

		sts		= HPDI32_RX_ENABLE__NO(dev);
		status	= (status == GSC_SUCCESS) ? sts : status;

		sts		= HPDI32_TX_ENABLE__YES(dev);
		status	= (status == GSC_SUCCESS) ? sts : status;

		sts		= data_sim_tx(dev, io_bin);
		status	= (status == GSC_SUCCESS) ? sts : status;
	}
	else
	{
		// GPIO: enable
		// Tx: disable
		// Rx: enable

		sts		= HPDI32_TX_ENABLE__NO(dev);
		status	= (status == GSC_SUCCESS) ? sts : status;

		sts		= HPDI32_RX_ENABLE__YES(dev);
		status	= (status == GSC_SUCCESS) ? sts : status;
	}

	return(status);
}



//*****************************************************************************
U32 data_sim_rx(void* dev, U32* io_bin)
{
	unsigned long	fifo		= HPDI32_FIFO_STATUS_EMPTY;
	int				i;
	U32				io_bin_tmp	= 0;
	U32				status		= GSC_SUCCESS;
	U32				sts;

	sts		= HPDI32_FIFO_RESET__RX_YES(dev);
	status	= (status == GSC_SUCCESS) ? sts : status;

	for (i = 0; i < 20; i++)
	{
		sts		= HPDI32_FIFO_STATUS__RX_GET(dev, &fifo);
		status	= (status == GSC_SUCCESS) ? sts : status;

		if (fifo != HPDI32_FIFO_STATUS_EMPTY)
			break;

		// Wait a little bit for possible input.
		usleep(1000);
	}

	if (fifo == HPDI32_FIFO_STATUS_EMPTY)
	{
		sts		= GSC_WAIT_TIMEOUT;
		status	= (status == GSC_SUCCESS) ? sts : status;
	}
	else
	{
		sts		= hpdi32_reg_read(dev, HPDI32_FDR, &io_bin_tmp);
		status	= (status == GSC_SUCCESS) ? sts : status;
	}

	if (io_bin)
		io_bin[0]	= io_bin_tmp;

	return(status);
}



//*****************************************************************************
U32 data_sim_tx(void* dev, U32 io_bin)
{
	int	i;
	U32	status	= GSC_SUCCESS;
	U32	sts;

	for (i = 0; i <= TX_AE_DEF; i++)
	{
		sts		= hpdi32_reg_write(dev, HPDI32_FDR, io_bin);
		status	= (status == GSC_SUCCESS) ? sts : status;

		// We only need this once, but this way the output appears sooner.
		sts		= HPDI32_TX_FLOW_CONTROL__START(dev);
		status	= (status == GSC_SUCCESS) ? sts : status;
	}

	return(status);
}


