// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/HPDI32/HPDI32_Linux_sdk_6.x.x_GSC_DN/sbtest/misc.c $
// $Rev: 30385 $
// $Date: 2014-09-18 12:14:48 -0500 (Thu, 18 Sep 2014) $

#include <stdio.h>
#include <string.h>

#include "main.h"



// data types *****************************************************************

typedef struct
{
	unsigned long	set;
	unsigned long	get;
} set_get_t;

typedef struct
{
	const char*			name;
	U32					parm;
	const set_get_t*	list;
	size_t				size;
} set_get_list_t;



/******************************************************************************
*
*	Function:	_misc_parm_list_test
*
*	Purpose:
*
*		Perform a series of tests per the given data and list.
*
*	Arguments:
*
*		handle	The handle used to access the device under test.
*
*		set		The data set to act on.
*
*		verbose	Work verbosely?
*
*	Returned:
*
*		0		All went well.
*		> 0		The number of errors seen.
*
******************************************************************************/

static int _misc_parm_list_test(void* handle, const set_get_list_t* set, int verbose)
{
	int				errs	= 0;
	unsigned long	get;
	int				i;
	U32				status	= GSC_SUCCESS;

	if ((verbose) && (set->name))
		label_text(set->name);

	for (i = 0; i < (int) set->size; i++)
	{
		status	= hpdi32_config(handle,
								set->parm,
								0,
								set->list[i].set,
								&get);

		if ((status == GSC_SUCCESS) && (get != set->list[i].get))
			status	= GSC_FAILED;
	}

	hpdi32_reset(handle);
	errs	= (status == GSC_SUCCESS) ? 0 : 1;

	if (verbose == 0)
		;
	else if (set->name == NULL)
		;
	else if (errs)
		printf("FAIL <---\n");
	else
		printf("PASS\n");

	return(errs);
}



/******************************************************************************
*
*	Function:	_misc_board_jumpers_tests
*
*	Purpose:
*
*		Perform Miscallaneous Board Jumpers tests.
*
*	Arguments:
*
*		handle	The handle used to access the device under test.
*
*	Returned:
*
*		0		All went well.
*		> 0		The number of errors seen.
*
******************************************************************************/

static int _misc_board_jumpers_tests(void* handle)
{
	int				errs	= 0;
	unsigned long	get;
	U32				status	= GSC_SUCCESS;

	label_text("Board Jumpers");

	status	= hpdi32_config(handle,
							HPDI32_MISC_BOARD_JUMPERS,
							0,
							GSC_NO_CHANGE,
							&get);
	hpdi32_reset(handle);
	errs	= (status == GSC_SUCCESS) ? 0 : 1;

	if (errs)
		printf("FAIL <---\n");
	else
		printf("PASS\n");

	return(errs);
}



/******************************************************************************
*
*	Function:	_misc_favor_tx_tests
*
*	Purpose:
*
*		Perform Miscellaneous Favor Tx tests.
*
*	Arguments:
*
*		handle	The handle used to access the device under test.
*
*
*	Returned:
*
*		0		All went well.
*		> 0		The number of errors seen.
*
******************************************************************************/

static int _misc_favor_tx_tests(void* handle)
{
	static const set_get_t	list[]	=
	{
		// set							get
		{ HPDI32_MISC_FAVOR_TX_DISABLE,	HPDI32_MISC_FAVOR_TX_DISABLE	},
		{ GSC_NO_CHANGE,				HPDI32_MISC_FAVOR_TX_DISABLE	},
		{ GSC_DEFAULT,					HPDI32_MISC_FAVOR_TX_DEFAULT	},
		{ HPDI32_MISC_FAVOR_TX_ENABLE,	HPDI32_MISC_FAVOR_TX_ENABLE		},
		{ GSC_NO_CHANGE,				HPDI32_MISC_FAVOR_TX_ENABLE		},
		{ GSC_DEFAULT,					HPDI32_MISC_FAVOR_TX_DEFAULT	},
		{ HPDI32_MISC_FAVOR_TX_DEFAULT,	HPDI32_MISC_FAVOR_TX_DEFAULT	},
		{ GSC_NO_CHANGE,				HPDI32_MISC_FAVOR_TX_DEFAULT	},
		{ GSC_DEFAULT,					HPDI32_MISC_FAVOR_TX_DEFAULT	}
	};

	static const set_get_list_t	set	=
	{
		/* name	*/	NULL,
		/* parm	*/	HPDI32_MISC_FAVOR_TX,
		/* list	*/	list,
		/* size	*/	ARRAY_ELEMENTS(list)
	};

	unsigned long	after	= 0;
	unsigned long	before	= 0;
	int				errs;
	U32				status;

	label_text("Favor Tx");
	errs	= _misc_parm_list_test(handle, &set, 1);

	if (errs == 0)
	{
		printf(".");
		status	= hpdi32_config(handle,
								HPDI32_MISC_FAVOR_TX,
								0,
								HPDI32_MISC_FAVOR_TX_DISABLE,
								NULL);

		if (status == GSC_SUCCESS)
		{
			status	= hpdi32_config(handle,
									HPDI32_IO_DMA_CHANNEL_SEL,
									HPDI32_WHICH_TX,
									GSC_DEFAULT,
									&before);
		}

		if (status == GSC_SUCCESS)
		{
			status	= hpdi32_config(handle,
									HPDI32_MISC_FAVOR_TX,
									0,
									HPDI32_MISC_FAVOR_TX_ENABLE,
									NULL);
		}

		if (status == GSC_SUCCESS)
		{
			status	= hpdi32_config(handle,
									HPDI32_IO_DMA_CHANNEL_SEL,
									HPDI32_WHICH_TX,
									GSC_DEFAULT,
									&after);
		}

		if ((before != HPDI32_IO_DMA_CHANNEL_SEL_RX_DEFAULT) ||
			(after != HPDI32_IO_DMA_CHANNEL_SEL_TX_DEFAULT))
		{
			status	= GSC_FAILED;
		}

		errs	= (status == GSC_SUCCESS) ? 0 : 1;

	}

	if (errs)
		printf("FAIL <---\n");
	else
		printf("PASS\n");

	return(errs);
}



/******************************************************************************
*
*	Function:	_misc_features_tests
*
*	Purpose:
*
*		Perform Miscellaneous Features tests.
*
*	Arguments:
*
*		handle	The handle used to access the device under test.
*
*	Returned:
*
*		0		All went well.
*		> 0		The number of errors seen.
*
******************************************************************************/

static int _misc_features_tests(void* handle)
{
	static set_get_t	list[]	=
	{
		// set	get
		{ 0,	0	}
	};

	static const set_get_list_t	set	=
	{
		/* name	*/	NULL,
		/* parm	*/	HPDI32_MISC_FEATURES,
		/* list	*/	list,
		/* size	*/	ARRAY_ELEMENTS(list)
	};

	int	errs	= 0;
	U32	frr;
	U32	fsr;
	U32	status;

	label_text("Features");
	status	= hpdi32_reg_read(handle, HPDI32_FRR, &frr);
	errs	= (status == GSC_SUCCESS) ? errs : 1;
	status	= hpdi32_reg_read(handle, HPDI32_FSR, &fsr);
	errs	= (status == GSC_SUCCESS) ? errs : 1;


	// The NO CHANGE option, which is an unknown and unsupported features..

	if (status == GSC_SUCCESS)
		printf(".");

	list[0].set	= GSC_NO_CHANGE;
	list[0].get	= HPDI32_MISC_FEATURES_ABSENT;
	errs	|= _misc_parm_list_test(handle, &set, 0);

	// Feature Set Register

	if (status == GSC_SUCCESS)
		printf(".");

	if (frr & HPDI32_FRR_FSR)
	{
		list[0].get	= HPDI32_MISC_FEATURES_PRESENT;
	}
	else
	{
		list[0].get	= HPDI32_MISC_FEATURES_ABSENT;
		fsr			= 0;
	}

	list[0].set	= HPDI32_MISC_FEATURES_FSR;
	errs		+= _misc_parm_list_test(handle, &set, 0);

	// FIFO Size Registers

	if (status == GSC_SUCCESS)
		printf(".");

	if (fsr & HPDI32_FSR_TFSR_RFSR)
		list[0].get	= HPDI32_MISC_FEATURES_PRESENT;
	else
		list[0].get	= HPDI32_MISC_FEATURES_ABSENT;

	list[0].set	= HPDI32_MISC_FEATURES_FIFO_SIZE;
	errs		+= _misc_parm_list_test(handle, &set, 0);

	// Interrupt Configuration Registers

	if (status == GSC_SUCCESS)
		printf(".");

	if (fsr & HPDI32_FSR_IELR_IHLR)
		list[0].get	= HPDI32_MISC_FEATURES_PRESENT;
	else
		list[0].get	= HPDI32_MISC_FEATURES_ABSENT;

	list[0].set	= HPDI32_MISC_FEATURES_ICR;
	errs		+= _misc_parm_list_test(handle, &set, 0);

	// GPIO 0-5 Cable Functionality

	if (status == GSC_SUCCESS)
		printf(".");

	if (fsr & HPDI32_FSR_GPIO_0_5)
		list[0].get	= HPDI32_MISC_FEATURES_PRESENT;
	else
		list[0].get	= HPDI32_MISC_FEATURES_ABSENT;

	list[0].set	= HPDI32_MISC_FEATURES_GPIO_0_5;
	errs		+= _misc_parm_list_test(handle, &set, 0);

	// GPIO 6 Cable Functionality

	if (status == GSC_SUCCESS)
		printf(".");

	if (fsr & HPDI32_FSR_GPIO6_TXAS)
		list[0].get	= HPDI32_MISC_FEATURES_PRESENT;
	else
		list[0].get	= HPDI32_MISC_FEATURES_ABSENT;

	list[0].set	= HPDI32_MISC_FEATURES_GPIO_6;
	errs		+= _misc_parm_list_test(handle, &set, 0);

	// DMA Channel 1

	if (status == GSC_SUCCESS)
		printf(".");

	if (fsr & HPDI32_FSR_DMDMA_CH1)
		list[0].get	= HPDI32_MISC_FEATURES_PRESENT;
	else
		list[0].get	= HPDI32_MISC_FEATURES_ABSENT;

	list[0].set	= HPDI32_MISC_FEATURES_DMA_CH1;
	errs		+= _misc_parm_list_test(handle, &set, 0);

	// Tx/Rx Over/Under Run bits

	if (status == GSC_SUCCESS)
		printf(".");

	if (fsr & HPDI32_FSR_OVR_UNDR_RUN)
		list[0].get	= HPDI32_MISC_FEATURES_PRESENT;
	else
		list[0].get	= HPDI32_MISC_FEATURES_ABSENT;

	list[0].set	= HPDI32_MISC_FEATURES_OVR_UNDR_RUN;
	errs		+= _misc_parm_list_test(handle, &set, 0);

	errs	= errs ? 1 : 0;

	// Tx Auto Stop (Tx Start Auto Clear Disable)

	if (status == GSC_SUCCESS)
		printf(".");

	if (fsr & HPDI32_FSR_GPIO6_TXAS)
		list[0].get	= HPDI32_MISC_FEATURES_PRESENT;
	else
		list[0].get	= HPDI32_MISC_FEATURES_ABSENT;

	list[0].set	= HPDI32_MISC_FEATURES_TX_AUTO_STOP;
	errs		+= _misc_parm_list_test(handle, &set, 0);

	errs	= errs ? 1 : 0;

	if (errs)
		printf("FAIL <---\n");
	else
		printf("PASS\n");

	return(errs);
}



/******************************************************************************
*
*	Function:	_misc_map_gsc_regs_tests
*
*	Purpose:
*
*		Perform Miscellaneous Map GSC Registers tests.
*
*	Arguments:
*
*		handle	The handle used to access the device under test.
*
*	Returned:
*
*		0		All went well.
*		> 0		The number of errors seen.
*
******************************************************************************/

static int _misc_map_gsc_regs_tests(void* handle)
{
	static const set_get_t	list[]	=
	{
		// set								get
		{ HPDI32_MISC_MAP_GSC_REGS_DISABLE,	HPDI32_MISC_MAP_GSC_REGS_DISABLE	},
		{ GSC_NO_CHANGE,					HPDI32_MISC_MAP_GSC_REGS_DISABLE	},
		{ GSC_DEFAULT,						HPDI32_MISC_MAP_GSC_REGS_DEFAULT	},
		{ HPDI32_MISC_MAP_GSC_REGS_ENABLE,	HPDI32_MISC_MAP_GSC_REGS_ENABLE		},
		{ GSC_NO_CHANGE,					HPDI32_MISC_MAP_GSC_REGS_ENABLE		},
		{ GSC_DEFAULT,						HPDI32_MISC_MAP_GSC_REGS_DEFAULT	},
		{ HPDI32_MISC_MAP_GSC_REGS_DEFAULT,	HPDI32_MISC_MAP_GSC_REGS_DEFAULT	},
		{ GSC_NO_CHANGE,					HPDI32_MISC_MAP_GSC_REGS_DEFAULT	},
		{ GSC_DEFAULT,						HPDI32_MISC_MAP_GSC_REGS_DEFAULT	}
	};

	static const set_get_list_t	set	=
	{
		/* name	*/	NULL,
		/* parm	*/	HPDI32_MISC_MAP_GSC_REGS,
		/* list	*/	list,
		/* size	*/	ARRAY_ELEMENTS(list)
	};

	unsigned long	after	= 0;
	unsigned long	before	= 0;
	int				errs;
	U32				status;

	label_text("Map GSC Registers");
	errs	= _misc_parm_list_test(handle, &set, 1);

	if (errs == 0)
	{
		printf(".");
		status	= hpdi32_config(handle,
								HPDI32_MISC_MAP_GSC_REGS,
								0,
								HPDI32_MISC_MAP_GSC_REGS_DISABLE,
								NULL);

		if (status == GSC_SUCCESS)
		{
			status	= hpdi32_config(handle,
									HPDI32_MISC_MAP_GSC_REGS_PTR,
									0,
									GSC_NO_CHANGE,
									&before);
		}

		if (status == GSC_SUCCESS)
		{
			status	= hpdi32_config(handle,
									HPDI32_MISC_MAP_GSC_REGS,
									0,
									HPDI32_MISC_MAP_GSC_REGS_DISABLE,
									NULL);
		}

		if (status == GSC_SUCCESS)
		{
			status	= hpdi32_config(handle,
									HPDI32_MISC_MAP_GSC_REGS_PTR,
									0,
									GSC_NO_CHANGE,
									&after);
		}

		if ((before) || (after != 0))
			status	= GSC_FAILED;

		errs	= (status == GSC_SUCCESS) ? 0 : 1;

	}

	if (errs)
		printf("FAIL <---\n");
	else
		printf("PASS\n");

	return(errs);
}



/******************************************************************************
*
*	Function:	_misc_map_plx_regs_tests
*
*	Purpose:
*
*		Perform Miscellaneous Map PLX Registers tests.
*
*	Arguments:
*
*		handle	The handle used to access the device under test.
*
*	Returned:
*
*		0		All went well.
*		> 0		The number of errors seen.
*
******************************************************************************/

static int _misc_map_plx_regs_tests(void* handle)
{
	static const set_get_t	list[]	=
	{
		// set								get
		{ HPDI32_MISC_MAP_PLX_REGS_DISABLE,	HPDI32_MISC_MAP_PLX_REGS_DISABLE	},
		{ GSC_NO_CHANGE,					HPDI32_MISC_MAP_PLX_REGS_DISABLE	},
		{ GSC_DEFAULT,						HPDI32_MISC_MAP_PLX_REGS_DEFAULT	},
		{ HPDI32_MISC_MAP_PLX_REGS_ENABLE,	HPDI32_MISC_MAP_PLX_REGS_ENABLE		},
		{ GSC_NO_CHANGE,					HPDI32_MISC_MAP_PLX_REGS_ENABLE		},
		{ GSC_DEFAULT,						HPDI32_MISC_MAP_PLX_REGS_DEFAULT	},
		{ HPDI32_MISC_MAP_PLX_REGS_DEFAULT,	HPDI32_MISC_MAP_PLX_REGS_DEFAULT	},
		{ GSC_NO_CHANGE,					HPDI32_MISC_MAP_PLX_REGS_DEFAULT	},
		{ GSC_DEFAULT,						HPDI32_MISC_MAP_PLX_REGS_DEFAULT	}
	};

	static const set_get_list_t	set	=
	{
		/* name	*/	"Map PLX Registers",
		/* parm	*/	HPDI32_MISC_MAP_PLX_REGS,
		/* list	*/	list,
		/* size	*/	ARRAY_ELEMENTS(list)
	};

	int	errs;

	errs	= _misc_parm_list_test(handle, &set, 1);
	return(errs);
}



/******************************************************************************
*
*	Function:	_misc_pci_bus_width_tests
*
*	Purpose:
*
*		Perform Miscellaneous PCi Bus Width tests.
*
*	Arguments:
*
*		handle	The handle used to access the device under test.
*
*	Returned:
*
*		0		All went well.
*		> 0		The number of errors seen.
*
******************************************************************************/

static int _misc_pci_bus_width_tests(void* handle)
{
	static set_get_t	list[]	=
	{
		// set				get
		{ GSC_NO_CHANGE,	0	}
	};

	static const set_get_list_t	set	=
	{
		/* name	*/	NULL,
		/* parm	*/	HPDI32_MISC_PCI_BUS_WIDTH,
		/* list	*/	list,
		/* size	*/	ARRAY_ELEMENTS(list)
	};

	U32	did;
	int	errs;
	U32	status;

	label_text("PCI Bus Width");
	status	= hpdi32_reg_read(handle, GSC_PCI_9080_DIDR, &did);

	if (status != GSC_SUCCESS)
		;
	else if (did == HPDI32_DEVICE_ID_32)
		list[0].get	= HPDI32_MISC_PCI_BUS_WIDTH_32;
	else if (did == HPDI32_DEVICE_ID_64)
		list[0].get	= HPDI32_MISC_PCI_BUS_WIDTH_64;
	else
		status	= GSC_FAILED;

	if (status != GSC_SUCCESS)
		errs	= 1;
	else
		errs	= _misc_parm_list_test(handle, &set, 1);

	if (errs)
		printf("FAIL <---\n");
	else
		printf("PASS\n");

	return(errs);
}



/******************************************************************************
*
*	Function:	_misc_strict_arguments_tests
*
*	Purpose:
*
*		Perform Miscellaneous Strict Arguments tests.
*
*	Arguments:
*
*		handle	The handle used to access the device under test.
*
*	Returned:
*
*		0		All went well.
*		> 0		The number of errors seen.
*
******************************************************************************/

static int _misc_strict_arguments_tests(void* handle)
{
	static const set_get_t	list[]	=
	{
		// set									get
		{ HPDI32_MISC_STRICT_ARGUMENTS_DISABLE,	HPDI32_MISC_STRICT_ARGUMENTS_DISABLE	},
		{ GSC_NO_CHANGE,						HPDI32_MISC_STRICT_ARGUMENTS_DISABLE	},
		{ GSC_DEFAULT,							HPDI32_MISC_STRICT_ARGUMENTS_DEFAULT	},
		{ HPDI32_MISC_STRICT_ARGUMENTS_ENABLE,	HPDI32_MISC_STRICT_ARGUMENTS_ENABLE		},
		{ GSC_NO_CHANGE,						HPDI32_MISC_STRICT_ARGUMENTS_ENABLE		},
		{ GSC_DEFAULT,							HPDI32_MISC_STRICT_ARGUMENTS_DEFAULT	},
		{ HPDI32_MISC_STRICT_ARGUMENTS_DEFAULT,	HPDI32_MISC_STRICT_ARGUMENTS_DEFAULT	},
		{ GSC_NO_CHANGE,						HPDI32_MISC_STRICT_ARGUMENTS_DEFAULT	},
		{ GSC_DEFAULT,							HPDI32_MISC_STRICT_ARGUMENTS_DEFAULT	}
	};

	static const set_get_list_t	set	=
	{
		/* name	*/	NULL,
		/* parm	*/	HPDI32_MISC_STRICT_ARGUMENTS,
		/* list	*/	list,
		/* size	*/	ARRAY_ELEMENTS(list)
	};

	U32	after;
	U32	before;
	int	errs;
	U32	status;

	label_text("Strict Arguments");
	errs	= _misc_parm_list_test(handle, &set, 1);

	if (errs == 0)
	{
		printf(".");
		status	= hpdi32_config(handle,
								HPDI32_MISC_STRICT_ARGUMENTS,
								0,
								HPDI32_MISC_STRICT_ARGUMENTS_ENABLE,
								NULL);

		before	= hpdi32_config(handle,
								HPDI32_MISC_STRICT_ARGUMENTS,
								~0,
								HPDI32_MISC_STRICT_ARGUMENTS_ENABLE,
								NULL);

		if (status == GSC_SUCCESS)
		{
			status	= hpdi32_config(handle,
									HPDI32_MISC_STRICT_ARGUMENTS,
									0,
									HPDI32_MISC_STRICT_ARGUMENTS_DISABLE,
									NULL);
		}

		after	= hpdi32_config(handle,
								HPDI32_MISC_STRICT_ARGUMENTS,
								~0,
								HPDI32_MISC_STRICT_ARGUMENTS_DISABLE,
								NULL);

		if ((before == GSC_SUCCESS) || (after != GSC_SUCCESS))
			status	= GSC_FAILED;

		errs	= (status == GSC_SUCCESS) ? 0 : 1;

	}

	if (errs)
		printf("FAIL <---\n");
	else
		printf("PASS\n");

	return(errs);
}



/******************************************************************************
*
*	Function:	_misc_tx_rx_tri_state_tests
*
*	Purpose:
*
*		Perform Miscellaneous Tx/Rx Tri-State tests.
*
*	Arguments:
*
*		handle	The handle used to access the device under test.
*
*	Returned:
*
*		0		All went well.
*		> 0		The number of errors seen.
*
******************************************************************************/

static int _misc_tx_rx_tri_state_tests(void* handle)
{
	static const set_get_t	list[]	=
	{
		// set									get
		{ HPDI32_MISC_TX_RX_TRI_STATE_DISABLE,	HPDI32_MISC_TX_RX_TRI_STATE_DISABLE	},
		{ GSC_NO_CHANGE,						HPDI32_MISC_TX_RX_TRI_STATE_DISABLE	},
		{ GSC_DEFAULT,							HPDI32_MISC_TX_RX_TRI_STATE_DEFAULT	},
		{ HPDI32_MISC_TX_RX_TRI_STATE_ENABLE,	HPDI32_MISC_TX_RX_TRI_STATE_ENABLE	},
		{ GSC_NO_CHANGE,						HPDI32_MISC_TX_RX_TRI_STATE_ENABLE	},
		{ GSC_DEFAULT,							HPDI32_MISC_TX_RX_TRI_STATE_DEFAULT	},
		{ HPDI32_MISC_TX_RX_TRI_STATE_DEFAULT,	HPDI32_MISC_TX_RX_TRI_STATE_DEFAULT	},
		{ GSC_NO_CHANGE,						HPDI32_MISC_TX_RX_TRI_STATE_DEFAULT	},
		{ GSC_DEFAULT,							HPDI32_MISC_TX_RX_TRI_STATE_DEFAULT	}
	};

	static const set_get_list_t	set	=
	{
		/* name	*/	NULL,
		/* parm	*/	HPDI32_MISC_TX_RX_TRI_STATE,
		/* list	*/	list,
		/* size	*/	ARRAY_ELEMENTS(list)
	};

	U32	after	= 0;
	U32	before	= 0;
	int	errs;
	U32	status;

	label_text("Tx/Rx Tri-State");
	errs	= _misc_parm_list_test(handle, &set, 1);

	if (errs == 0)
	{
		printf(".");
		status	= hpdi32_config(handle,
								HPDI32_MISC_TX_RX_TRI_STATE,
								0,
								HPDI32_MISC_TX_RX_TRI_STATE_ENABLE,
								NULL);

		if (status == GSC_SUCCESS)
			status	= hpdi32_reg_read(handle, HPDI32_BCR, &before);

		if (status == GSC_SUCCESS)
		{
			status	= hpdi32_config(handle,
									HPDI32_MISC_TX_RX_TRI_STATE,
									0,
									HPDI32_MISC_TX_RX_TRI_STATE_DISABLE,
									NULL);
		}

		if (status == GSC_SUCCESS)
			status	= hpdi32_reg_read(handle, HPDI32_BCR, &after);

		if (((before & HPDI32_BCR_TEST_MODE) != HPDI32_BCR_TEST_MODE) ||
			((after & HPDI32_BCR_TEST_MODE) == HPDI32_BCR_TEST_MODE))
		{
			status	= GSC_FAILED;
		}

		errs	= (status == GSC_SUCCESS) ? 0 : 1;

	}

	if (errs)
		printf("FAIL <---\n");
	else
		printf("PASS\n");

	return(errs);
}



/******************************************************************************
*
*	Function:	misc_config_tests
*
*	Purpose:
*
*		Perform Miscellaneous configuration tests.
*
*	Arguments:
*
*		handle	The handle used to access the device under test.
*
*	Returned:
*
*		0		All went well.
*		> 0		The number of errors seen.
*
******************************************************************************/

int misc_config_tests(void* handle)
{
	int	errs	= 0;

	label_text("Misc. Configuration");
	printf("\n");
	label_level_inc();

	errs	+= _misc_board_jumpers_tests(handle);
	errs	+= _misc_favor_tx_tests(handle);
	errs	+= _misc_features_tests(handle);
	errs	+= _misc_map_gsc_regs_tests(handle);
	errs	+= _misc_map_plx_regs_tests(handle);
	errs	+= _misc_pci_bus_width_tests(handle);
	errs	+= _misc_strict_arguments_tests(handle);
	errs	+= _misc_tx_rx_tri_state_tests(handle);

	label_level_dec();
	return(errs);
}


