// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/HPDI32/HPDI32_Linux_sdk_6.x.x_GSC_DN/utils/util_issues.c $
// $Rev: 30390 $
// $Date: 2014-09-18 12:16:55 -0500 (Thu, 18 Sep 2014) $

#include <stdio.h>
#include <string.h>

#include "hpdi32_utils.h"



/******************************************************************************
*
*	Function:	_issue_gsc_reg_mapping
*
*	Purpose:
*
*		See if the mapping of GSC firmware registers affects operation.
*
*	Arguments:
*
*		handle	The handle of the board to access.
*
*	Returned:
*
*		0		All went well.
*		> 0		The number of errors encountered.
*
******************************************************************************/

static int _issue_gsc_reg_mapping(void* handle)
{
	int				errs	= 0;
	unsigned long	get		= 0;
	U32				status;

	label_text("GSC Register Mapping");
	status	= hpdi32_init(handle);
	errs	+= label_fail_func(1, "hpdi32_init", status);

	if (errs == 0)
	{
		status	= HPDI32_MISC_MAP_GSC_REGS__GET(handle, &get);
		errs	+= label_fail_func(1, "HPDI32_MISC_MAP_GSC_REGS__GET", status);
	}

	if (errs)
		;
	else if (get == HPDI32_MISC_MAP_GSC_REGS_DISABLE)
		printf("Disabled  (PERFORMANCE WILL BE DEGRADED)\n");
	else
		printf("OK\n");

	return(errs);
}



/******************************************************************************
*
*	Function:	_issue_plx_reg_mapping
*
*	Purpose:
*
*		See if the mapping of PLX feature set registers affects operation.
*
*	Arguments:
*
*		handle	The handle of the board to access.
*
*	Returned:
*
*		0		All went well.
*		> 0		The number of errors encountered.
*
******************************************************************************/

static int _issue_plx_reg_mapping(void* handle)
{
	int				errs	= 0;
	unsigned long	get		= 0;
	U32				status;

	label_text("PLX Register Mapping");
	status	= hpdi32_init(handle);
	errs	+= label_fail_func(1, "hpdi32_init", status);

	if (errs == 0)
	{
		status	= HPDI32_MISC_MAP_PLX_REGS__GET(handle, &get);
		errs	+= label_fail_func(errs == 0, "HPDI32_MISC_MAP_PLX_REGS__GET", status);
	}

	if (errs)
		;
	else if (get == HPDI32_MISC_MAP_PLX_REGS_DISABLE)
		printf("Disabled  (PERFORMANCE WILL BE DEGRADED)\n");
	else
		printf("OK\n");

	return(errs);
}



/******************************************************************************
*
*	Function:	_issue_tx_fifo_size
*
*	Purpose:
*
*		See if the Tx FIFO size affects operation.
*
*	Arguments:
*
*		handle		The handle of the board to access.
*
*		size		Record the FIFO size here.
*
*	Returned:
*
*		0			All went well.
*		> 0			The number of errors encountered.
*
******************************************************************************/

static int _issue_tx_fifo_size(void* handle, long* size)
{
	char			buf[32];
	int				errs	= 0;
	unsigned long	feature	= 0;
	U32				status;

	label_text("Tx FIFO Size");
	status	= HPDI32_FIFO_SIZE__TX_GET(handle, (unsigned long*) size);
	errs	+= label_fail_func(1, "HPDI32_FIFO_SIZE__TX_GET", status);

	if (errs == 0)
	{
		status	= HPDI32_MISC_FEATURES__FIFO_SIZE(handle, &feature);
		errs	+= label_fail_func(errs == 0, "HPDI32_MISC_FEATURES__FIFO_SIZE", status);
	}

	if (errs)
	{
		size[0]	= -1;
	}
	else if (feature == HPDI32_MISC_FEATURES_ABSENT)
	{
		printf("Unknown  (PERFORMANCE MAY BE DEGRADED)\n");
		size[0]	= -1;
	}
	else if (size[0] == 0)
	{
		printf("NO FIFOS INSTALLED, TRANSMISSION NOT POSSIBLE\n");
	}
	else if (size[0] >= (128L * 1024))
	{
		format_s32(size[0], 1024, buf, sizeof(buf));
		printf("OK  (%s samples)\n", buf);
	}
	else
	{
		format_s32(size[0], 1024, buf, sizeof(buf));
		printf("OK  (%s samples, less than optimal)\n", buf);
	}

	return(errs);
}



/******************************************************************************
*
*	Function:	_issue_rx_fifo_size
*
*	Purpose:
*
*		See if the Rx FIFO size affects operation.
*
*	Arguments:
*
*		handle	The handle of the board to access.
*
*		size		Record the FIFO size here. -1 = unknown
*
*	Returned:
*
*		0		All went well.
*		> 0		The number of errors encountered.
*
******************************************************************************/

static int _issue_rx_fifo_size(void* handle, long* size)
{
	char			buf[32];
	int				errs	= 0;
	unsigned long	feature	= 0;
	U32				status;

	label_text("Rx FIFO Size");
	status	= HPDI32_FIFO_SIZE__RX_GET(handle, (unsigned long*) size);
	errs	+= label_fail_func(1, "HPDI32_FIFO_SIZE__RX_GET", status);

	if (errs == 0)
	{
		status	= HPDI32_MISC_FEATURES__FIFO_SIZE(handle, &feature);
		errs	+= label_fail_func(errs == 0, "HPDI32_MISC_FEATURES__FIFO_SIZE", status);
	}

	if (errs)
	{
	}
	else if (feature == HPDI32_MISC_FEATURES_ABSENT)
	{
		printf("Unknown  (PERFORMANCE MAY BE DEGRADED)\n");
		size[0]	= -1;
	}
	else if (size[0] == 0)
	{
		printf("NO FIFOS INSTALLED, RECEPTION NOT POSSIBLE\n");
	}
	else if (size[0] >= (128L * 1024))
	{
		format_s32(size[0], 1024, buf, sizeof(buf));
		printf("OK  (%s samples)\n", buf);
	}
	else
	{
		format_s32(size[0], 1024, buf, sizeof(buf));
		printf("OK  (%s samples, less than optimal)\n", buf);
	}

	return(errs);
}



/******************************************************************************
*
*	Function:	_issue_tx_auto_stop
*
*	Purpose:
*
*		See if the Tx Auto Stop driver feature affects operation. This is the
*		API's presentation of the Tx Start Auto Clear Disable firmware
*		feature.
*
*	Arguments:
*
*		handle	The handle of the board to access.
*
*	Returned:
*
*		0		All went well.
*		> 0		The number of errors encountered.
*
******************************************************************************/

static int _issue_tx_auto_stop(void* handle)
{
	int				errs	= 0;
	unsigned long	get;
	U32				status;

	label_text("Tx Auto Stop");
	status	= HPDI32_MISC_FEATURES__TX_AUTO_STOP(handle, &get);
	errs	+= label_fail_func(1, "HPDI32_MISC_FEATURES__TX_AUTO_STOP", status);

	if (errs)
		;
	else if (get == HPDI32_MISC_FEATURES_ABSENT)
		printf("Not In Firmware  (TX PERFORMANCE MAY BE DEGRADED)\n");
	else
		printf("OK\n");

	return(errs);
}



/******************************************************************************
*
*	Function:	_issue_single_cycle_disable
*
*	Purpose:
*
*		See if the Rx DMDMA Single Cycle Disable feature affects operation.
*
*	Arguments:
*
*		handle	The handle of the board to access.
*
*	Returned:
*
*		0		All went well.
*		> 0		The number of errors encountered.
*
******************************************************************************/

static int _issue_single_cycle_disable(void* handle)
{
	U32				did		= 0;
	int				errs	= 0;
	unsigned long	get;
	U32				status;

	label_text("Single Cycle Disable");
	status	= HPDI32_MISC_FEATURES__1_CYCLE_DISABLE(handle, &get);
	errs	+= label_fail_func(1, "HPDI32_MISC_FEATURES__1_CYCLE_DISABLE", status);

	if (errs == 0)
	{
		status	= hpdi32_reg_read(handle, GSC_PCI_9080_DIDR, &did);
		errs	+= label_fail_func(errs == 0, "hpdi32_reg_read", status);
	}

	if (errs)
		;
	else if (get == HPDI32_MISC_FEATURES_PRESENT)
		printf("OK\n");
	else if (did == HPDI32_DEVICE_ID_32)
		printf("Not In Firmware  (8/16-bit DMDMA may over/under run FIFOs)\n");
	else if (did == HPDI32_DEVICE_ID_64)
		printf("Not In Firmware  (DMDMA may over/under run FIFOs)\n");
	else
		errs	+= label_fail_text(1, "not in firmware");

	return(errs);
}



/******************************************************************************
*
*	Function:	operation_issues
*
*	Purpose:
*
*		Report any items known to hinder performance.
*
*	Arguments:
*
*		handle		The handle of the board to access.
*
*		tx_size		Record the Tx FIFO size here. -1 = unknown
*
*		rx_size		Record the Rx FIFO size here. -1 = unknown
*
*	Returned:
*
*		0			All went well.
*		> 0			The number of errors encountered.
*
******************************************************************************/

int operation_issues(void* handle, long* tx_size, long* rx_size)
{
	int	errs	= 0;

	label_text("Operation Issues");
	printf("\n");
	label_level_inc();

	errs	+= host_issue_os();
	errs	+= _issue_gsc_reg_mapping(handle);
	errs	+= _issue_plx_reg_mapping(handle);
	errs	+= _issue_tx_fifo_size(handle, tx_size);
	errs	+= _issue_rx_fifo_size(handle, rx_size);
	errs	+= _issue_tx_auto_stop(handle);
	errs	+= _issue_single_cycle_disable(handle);

	label_level_dec();
	return(errs);
}


